/*
 *   Copyright 2013 INFTEL - Proveedor de Servicios de Aplicación
 *   (www.inftel.com.mx). All Rights Reserved (Todos los derechos reservados).
 * 
 *   Copyright 2013 Santos Zatarain Vera (santoszv _at_ inftel.com.mx).
 *   All Rights Reserved (Todos los derechos reservados).
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
package mx.com.inftel.validators;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;
import javax.validation.constraints.Pattern;

/**
 * Valida la Clave Única de Registro de Población (CURP). Validación de
 * secuencia bien formada únicamente.
 *
 * @author Santos Zatarain Vera `<santoszv@inftel.com.mx>`
 */
@Target({ElementType.METHOD, ElementType.FIELD, ElementType.ANNOTATION_TYPE})
@Retention(RetentionPolicy.RUNTIME)
@Constraint(validatedBy = {})
@Documented
@Pattern(regexp = "[A-Z]{4}[0-9]{6}[HM][A-Z]{2}[B-DF-HJ-NP-TV-Z]{3}[A-Z0-9][0-9]")
public @interface CURP {

    String message() default "{mx.com.inftel.validators.curp.malformado}";

    Class<?>[] groups() default {};

    Class<? extends Payload>[] payload() default {};

    /**
     * Valida si la CURP contiene palabras inconvenientes.
     */
    @Target({ElementType.METHOD, ElementType.FIELD, ElementType.ANNOTATION_TYPE})
    @Retention(RetentionPolicy.RUNTIME)
    @Constraint(validatedBy = PalabraInconvenienteValidator.class)
    @Documented
    public @interface PalabraInconveniente {

        String message() default "{mx.com.inftel.validators.curp.palabrainconveniente}";

        Class<?>[] groups() default {};

        Class<? extends Payload>[] payload() default {};
    }

    /**
     * Valida el dígito verificador de la CURP.
     */
    @Target({ElementType.METHOD, ElementType.FIELD, ElementType.ANNOTATION_TYPE})
    @Retention(RetentionPolicy.RUNTIME)
    @Constraint(validatedBy = DigitoVerificadorValidator.class)
    @Documented
    public @interface DigitoVerificador {

        String message() default "{mx.com.inftel.validators.curp.digitoverificador}";

        Class<?>[] groups() default {};

        Class<? extends Payload>[] payload() default {};
    }

    public static class PalabraInconvenienteValidator implements ConstraintValidator<PalabraInconveniente, String> {

        private static final String[] palabras;

        static {
            palabras = new String[]{
                "BACA", "BAKA",
                "BUEI", "BUEY",
                "CACA", "CACO", "CAGA", "CAGO", "CAKA", "CAKO",
                "COGE", "COGI", "COJA", "COJE", "COJI", "COJO", "COLA",
                "CULO",
                "FALO",
                "FETO",
                "GETA",
                "GUEI", "GUEY",
                "JOTO",
                "KACA", "KACO", "KAGA", "KAGO", "KAKA", "KAKO",
                "KOGE", "KOGI", "KOJA", "KOJE", "KOJI", "KOJO",
                "KOLA",
                "KULO",
                "LILO",
                "LOCA", "LOCO", "LOKA", "LOKO",
                "MAME", "MAMO",
                "MEAR", "MEAS", "MEON",
                "MIAR", "MION",
                "MOCO", "MOKO",
                "MULA", "MULO",
                "NACA", "NACO",
                "PEDA", "PEDO", "PENE",
                "PIPI", "PITO",
                "POPO",
                "PUTA", "PUTO",
                "QULO",
                "RATA",
                "ROBA", "ROBE", "ROBO",
                "RUIN",
                "SENO",
                "TETA",
                "VACA", "VAGA", "VAGO", "VAKA",
                "VUEI", "VUEY",
                "WUEI", "WUEY"};
        }

        @Override
        public void initialize(PalabraInconveniente constraintAnnotation) {
            // NO OPERATION
        }

        @Override
        public boolean isValid(String value, ConstraintValidatorContext context) {
            if (value == null) {
                return true;
            }
            for (String palabra : palabras) {
                if (value.startsWith(palabra)) {
                    return false;
                }
            }
            return true;
        }
    }

    public static class DigitoVerificadorValidator implements ConstraintValidator<DigitoVerificador, String> {

        private static final String valores = "0123456789ABCDEFGHIJKLMNÑOPQRSTUVWXYZ";

        @Override
        public void initialize(DigitoVerificador constraintAnnotation) {
            // NO OPERATION
        }

        @Override
        public boolean isValid(String value, ConstraintValidatorContext context) {
            if (value == null) {
                return true;
            }
            if (value.length() != 18) {
                return false;
            }

            int resultado = 0;
            for (int i = 0; i < 17; i++) {
                int valor = valores.indexOf(value.charAt(i));
                resultado += valor * (18 - i);
            }
            resultado = 10 - (resultado % 10);
            String digito;
            if (resultado == 10) {
                digito = "0";
            } else {
                digito = String.valueOf(resultado);
            }
            return value.endsWith(digito);
        }
    }
}
