/*
 *   Copyright 2013 INFTEL - Proveedor de Servicios de Aplicación
 *   (www.inftel.com.mx). All Rights Reserved (Todos los derechos reservados).
 * 
 *   Copyright 2013 Santos Zatarain Vera (santoszv _at_ inftel.com.mx).
 *   All Rights Reserved (Todos los derechos reservados).
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 */
package mx.com.inftel.validators;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;
import javax.validation.Constraint;
import javax.validation.ConstraintValidator;
import javax.validation.ConstraintValidatorContext;
import javax.validation.Payload;
import javax.validation.constraints.Pattern;

/**
 * Valida el Registro Federal del Contribuyente (RFC). Validación de secuencia
 * bien formada únicamente.
 *
 * @author Santos Zatarain Vera `<santoszv@inftel.com.mx>`
 */
@Target({ElementType.METHOD, ElementType.FIELD, ElementType.ANNOTATION_TYPE})
@Retention(RetentionPolicy.RUNTIME)
@Constraint(validatedBy = {})
@Documented
@Pattern(regexp = "[&A-Z\u00d1]{3,4}[0-9]{6}[1-9A-Z]{2}[0-9A]{1}")
public @interface RFC {

    String message() default "{mx.com.inftel.validators.rfc.malformado}";

    Class<?>[] groups() default {};

    Class<? extends Payload>[] payload() default {};

    /**
     * Valida si el RFC contiene palabras inconvenientes.
     */
    @Target({ElementType.METHOD, ElementType.FIELD, ElementType.ANNOTATION_TYPE})
    @Retention(RetentionPolicy.RUNTIME)
    @Constraint(validatedBy = PalabraInconvenienteValidator.class)
    @Documented
    public @interface PalabraInconveniente {

        String message() default "{mx.com.inftel.validators.rfc.palabrainconveniente}";

        Class<?>[] groups() default {};

        Class<? extends Payload>[] payload() default {};
    }

    /**
     * Valida el dígito verificador del RFC.
     */
    @Target({ElementType.METHOD, ElementType.FIELD, ElementType.ANNOTATION_TYPE})
    @Retention(RetentionPolicy.RUNTIME)
    @Constraint(validatedBy = DigitoVerificadorValidator.class)
    @Documented
    public @interface DigitoVerificador {

        String message() default "{mx.com.inftel.validators.rfc.digitoverificador}";

        Class<?>[] groups() default {};

        Class<? extends Payload>[] payload() default {};
    }

    public static class PalabraInconvenienteValidator implements ConstraintValidator<PalabraInconveniente, String> {

        private static final String[] palabras;

        static {
            palabras = new String[]{
                "BUEI", "BUEY",
                "CACA", "CACO", "CAGA", "CAGO", "CAKA",
                "COGE", "COJA", "COJE", "COJI", "COJO",
                "CULO",
                "FETO",
                "GUEY",
                "JOTO",
                "KACA", "KACO", "KAGA", "KAGO", "KAKA",
                "KOGE", "KOJO",
                "KULO",
                "MAME", "MAMO",
                "MEAR", "MEON",
                "MION",
                "MOCO",
                "MULA",
                "PEDA", "PEDO", "PENE",
                "PUTA", "PUTO",
                "QULO",
                "RATA",
                "RUIN"};
        }

        @Override
        public void initialize(PalabraInconveniente constraintAnnotation) {
            // NO OPERATION
        }

        @Override
        public boolean isValid(String value, ConstraintValidatorContext context) {
            if (value == null) {
                return true;
            }
            for (String palabra : palabras) {
                if (value.startsWith(palabra)) {
                    return false;
                }
            }
            return true;
        }
    }

    public static class DigitoVerificadorValidator implements ConstraintValidator<DigitoVerificador, String> {

        private static final String valores = "0123456789ABCDEFGHIJKLMN&OPQRSTUVWXYZ Ñ";

        @Override
        public void initialize(DigitoVerificador constraintAnnotation) {
            // NO OPERATION
        }

        @Override
        public boolean isValid(String value, ConstraintValidatorContext context) {
            if (value == null) {
                return true;
            }
            if (value.length() == 12) {
                value = " " + value;
            }
            if (value.length() != 13) {
                return false;
            }

            int resultado = 0;
            for (int i = 0; i < 12; i++) {
                int valor = valores.indexOf(value.charAt(11 - i));
                resultado += valor * (i + 2);
            }
            resultado = resultado % 11;
            String digito;
            if (resultado == 0) {
                digito = "0";
            } else if (resultado == 10) {
                digito = "A";
            } else {
                digito = String.valueOf(11 - resultado);
            }
            return value.endsWith(digito);
        }
    }
}
